@@ -19,6 +19,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include <string.h>
+#include <stdio.h>
+
 #include "window-action-menu.h"
 #include "private.h"
 
@@ -30,7 +33,8 @@ typedef enum
   SHADE,
   MOVE,
   RESIZE,
-  PIN
+  PIN,
+  MOVE_TO_WORKSPACE
 } WindowAction;
 
 typedef struct _ActionMenuData ActionMenuData;
@@ -46,6 +50,7 @@ struct _ActionMenuData
   GtkWidget *resize_item;
   GtkWidget *close_item;
   GtkWidget *pin_item;
+  GtkWidget *workspace_item;
   guint idle_handler;
 };
 
@@ -137,6 +142,15 @@ item_activated_callback (GtkWidget *menu_item,
       else
         wnck_window_pin (amd->window);
       break;
+    case MOVE_TO_WORKSPACE: {
+        int workspace_index;
+
+        workspace_index = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (menu_item), "workspace"));
+
+        wnck_window_move_to_workspace (amd->window,
+                                       wnck_screen_get_workspace (wnck_window_get_screen (amd->window),
+                                       workspace_index));
+       }
     }
 }
 
@@ -254,6 +268,10 @@ update_menu_state (ActionMenuData *amd)
 
   gtk_widget_set_sensitive (amd->resize_item,
                             (actions & WNCK_WINDOW_ACTION_RESIZE) != 0);
+
+  gtk_widget_set_sensitive (amd->workspace_item,
+                            (actions & WNCK_WINDOW_ACTION_CHANGE_WORKSPACE) != 0);
+
   
   return FALSE;
 }
@@ -321,6 +339,78 @@ amd_free (ActionMenuData *amd)
   g_free (amd);
 }
 
+static char *
+get_workspace_name_with_accel (WnckWindow *window,
+			       int index)
+{
+  const char *name;
+  int number;
+ 
+  name = wnck_workspace_get_name (wnck_screen_get_workspace (wnck_window_get_screen (window),
+				  index));
+
+  g_assert (name != NULL);
+
+  /*
+   * If the name is of the form "Workspace x" where x is an unsigned
+   * integer, insert a '_' before the number if it is less than 10 and
+   * return it
+   */
+  number = 0;
+  if (sscanf (name, _("Workspace %d"), &number) == 1) {
+      char *new_name;
+
+      /*
+       * Above name is a pointer into the Workspace struct. Here we make
+       * a copy copy so we can have our wicked way with it.
+       */
+      if (number == 10)
+        new_name = g_strdup_printf (_("Workspace 1_0"));
+      else
+        new_name = g_strdup_printf (_("Workspace %s%d"),
+                                    number < 10 ? "_" : "",
+                                    number);
+      return new_name;
+  }
+  else {
+      /*
+       * Otherwise this is just a normal name. Escape any _ characters so that
+       * the user's workspace names do not get mangled.  If the number is less
+       * than 10 we provide an accelerator.
+       */
+      char *new_name;
+      const char *source;
+      char *dest;
+
+      /*
+       * Assume the worst case, that every character is a _.  We also
+       * provide memory for " (_#)"
+       */
+      new_name = g_malloc0 (strlen (name) * 2 + 6 + 1);
+
+      /*
+       * Now iterate down the strings, adding '_' to escape as we go
+       */
+      dest = new_name;
+      source = name;
+      while (*source != '\0') {
+          if (*source == '_')
+            *dest++ = '_';
+          *dest++ = *source++;
+      }
+
+      /* People don't start at workstation 0, but workstation 1 */
+      if (index < 9) {
+          g_snprintf (dest, 6, " (_%d)", index + 1);
+      }
+      else if (index == 9) {
+          g_snprintf (dest, 6, " (_0)");
+      }
+
+      return new_name;
+  }
+}
+
 /**
  * wnck_create_window_action_menu:
  * @window: a #WnckWindow
@@ -332,10 +422,11 @@ amd_free (ActionMenuData *amd)
 GtkWidget*
 wnck_create_window_action_menu (WnckWindow *window)
 {
-  GtkWidget *menu;
+  GtkWidget *menu, *submenu;
   ActionMenuData *amd;
   GtkWidget *separator;
-  
+  int num_workspaces, present_workspace, i;
+ 
   _wnck_stock_icons_init ();
   
   amd = g_new0 (ActionMenuData, 1);
@@ -405,6 +496,40 @@ wnck_create_window_action_menu (WnckWindow *window)
                          amd->pin_item);
   set_item_stock (amd->pin_item, NULL);
   
+  amd->workspace_item = gtk_menu_item_new_with_mnemonic (_("Move to Another _Workspace")); 
+  gtk_widget_show (amd->workspace_item);
+
+  submenu = gtk_menu_new ();
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (amd->workspace_item), 
+			     submenu);
+
+  gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                         amd->workspace_item);
+
+  num_workspaces = wnck_screen_get_workspace_count (wnck_window_get_screen (amd->window));
+  present_workspace = wnck_workspace_get_number (wnck_window_get_workspace (amd->window));
+ 
+  for (i = 0; i < num_workspaces; i++) {
+    char *name,*label;
+    GtkWidget *mi;
+	
+    name = get_workspace_name_with_accel (amd->window, i);
+    label = g_strdup_printf (_("%s"), name);
+
+    mi = make_menu_item (amd, MOVE_TO_WORKSPACE);
+    g_object_set_data (G_OBJECT (mi), "workspace", GINT_TO_POINTER (i));
+
+    if (i == present_workspace)
+	gtk_widget_set_sensitive (mi, FALSE);
+
+    gtk_menu_shell_append (GTK_MENU_SHELL (submenu), mi);
+    set_item_text (mi, label);
+    set_item_stock (mi, NULL);
+
+    g_free (name);
+    g_free (label);	
+  }	
+		 
   g_signal_connect_object (G_OBJECT (amd->window), 
                            "state_changed",
                            G_CALLBACK (state_changed_callback),
